#include <iostream.h>
#include <fstream.h>    // ifstream
#include <stdlib.h>     // exit
#include <string.h>     // strcpy
#include "GraphAll.h"

void OpenInput ( ifstream &, char * );

void main ( int argc, char **argv )
{
   char       LineIn[255] = "";
   CGraphAll  Map;
   int        Option;
   ifstream   Inp;
   ofstream   Log("GraphLog.txt");

// Debug option:  program supports command-line argument of file name
   if (argc > 1)
   {
      strcpy (LineIn, argv[1]);
      cout << "Reading data from file " << LineIn << endl;
   }

   OpenInput (Inp, LineIn);
   Log << "Data read from " << LineIn << endl;

   Map.fillGraph(Inp);

   cout.precision(3);

   cout << "\nThe data file has " << Map.nCities() << " towns, and "
        << Map.nRoads()  << " roads connecting them.\n";

   while (1)
   {
      char Start[255], Finish[255];

      cout << "Start city (empty line exits):  ";
      cin.getline(Start, 255);
      if ( ! *Start )
         break;

      cout << "Menu:\n"
           << "    (1)  All paths\n"
           << "    (2)  Simple paths\n"
           << "    (3)  Hamiltonian paths\n"
           << "    (4)  Relaxed traveling salesman problem\n";
      do
      {
         cout << "Enter a number between 1 and 4:  ";
         cin  >> Option;  cin.ignore(255, '\n');
         if (cin.fail()) abort();
      }  while ( Option < 1 && Option > 4 );

      if ( Option < 3)
      {
         cout << "Final city (empty line exits):  ";
         cin.getline(Finish, 255);
         if ( ! *Finish )
            break;
      }
      switch ( Option )
      {
      case 1: Map.allPaths(Start, Finish); 
              break;
      case 2: Map.simplePaths(Start, Finish);
              break;
      case 3: Map.Hamiltonian(Start);
              break;
      case 4: Map.travelSales(Start);
              Log << "TravelSales ended with this sequence:\n\n";
              Map.edgeList(Log);
              break;
      }
   }

   cout << "\n\nPress ENTER to end program:  ";
   cin.ignore();
   Log.close();
}

void OpenInput ( ifstream &Inp, char *LineIn )
{
   if ( ! *LineIn )
   {
      cout << "Input file:  ";
      cin.getline(LineIn, 255);
   }

   Inp.open(LineIn, ios::in|ios::nocreate);

   while ( Inp.fail() )
   {
      Inp.clear();
      cout << "Open failed for " << LineIn
           << "\nPlease enter a valid file name:  ";
      cin.getline(LineIn, 255);
      Inp.open(LineIn, ios::in|ios::nocreate);
      if ( Inp.eof() )
         exit(-1);
   }
}
