#include <iostream.h>
#include <fstream.h>    // ifstream
#include <stdlib.h>     // exit
#include <string.h>     // strcpy
#include "Graph.h"

void OpenInput ( ifstream &, char * );

void main ( int argc, char **argv )
{
   char     LineIn[255] = "";
   CGraph   Map;
   ifstream Inp;
   ofstream Log("GraphLog.txt");

// Debug option:  program supports command-line argument of file name
   if (argc > 1)
   {
      strcpy (LineIn, argv[1]);
      cout << "Reading data from file " << LineIn << endl;
   }

   OpenInput (Inp, LineIn);
   Log << "Data read from " << LineIn << endl;

   Map.fillGraph(Inp);

   cout.precision(3);

   cout << "\nThe data file has " << Map.nCities() << " towns, and "
        << Map.nRoads()  << " roads connecting them.\n";

   if (argc > 2)
   {
      strcpy (LineIn, argv[2]);
      cout << "Starting from " << LineIn << endl;
   }

   while (!Map.validCity(LineIn))  // break on known city
   {
      cout << "Please enter the starting city:  ";
      cin.getline(LineIn, 255);
      if (! Map.validCity(LineIn))
         cout << "Unrecognized city.  ";
      if (cin.fail())
         exit(-1);
   }
   
   Log << "\nThe data file has " << Map.nCities() << " towns, and "
       << Map.nRoads()  << " roads connecting them.\n"
       << "Starting point:  " << LineIn << endl;

   cout.put('\n');

// Recursive depth 1st, followed by a listing of the edges traversed in order
   cout << "Recursive depth-first" << endl;
   Map.depthRecur(LineIn); Map.edgeList(cout); 
   Log << "\nRecursive depth-first\n";
   Map.edgeList(Log);

// Iterative depth 1st, followed by a listing of the edges traversed in order
   cout << "Iterative depth-first" << endl;
   Map.depth1st(LineIn);   Map.edgeList(cout);
   Log << "\nIterative depth-first\n";
   Map.edgeList(Log);

// Breadth First, followed by a listing of the edges traversed in order
   cout << "Breadth-first" << endl;
   Map.breadth1st(LineIn); Map.edgeList(cout);
   Log << "\nBreadth-first\n";
   Map.edgeList(Log);

// Priority First, followed by a listing of the edges traversed in order
   cout << "Prim's algorithm" << endl;
   Map.Prim(LineIn); Map.edgeList(cout);
   Log << "\nPrim's algorithm\n";
   Map.edgeList(Log);

// Dijstra's algorithm
   cout << "Dijkstra's algorith" << endl;
   Map.Dijkstra(LineIn); Map.edgeList(cout);
   Log << "\nDijkstra's algorithm\n";
   Map.edgeList(Log);

   cout << "\n\nPress ENTER to end program:  ";
   cin.ignore();
   Log.close();
}

void OpenInput ( ifstream &Inp, char *LineIn )
{
   if ( ! *LineIn )
   {
      cout << "Input file:  ";
      cin.getline(LineIn, 255);
   }

   Inp.open(LineIn);

   while ( Inp.fail() )
   {
      Inp.clear();
      cout << "Open failed for " << LineIn
           << "\nPlease enter a valid file name:  ";
      cin.getline(LineIn, 255);
      Inp.open(LineIn);
      if ( Inp.eof() )
         exit(-1);
   }
}
